# Using Amaze TDD

Documentation and examples for using Amaze TDD and writing tests during development.

## On this page...

* What is it?
* Requirements
    * Mac
    * Windows
    * Linux
* Browsers
* The `Amaze` object
* `.amazerc` configuration options
* Writing a test
    * TDD interface
    * Using jQuery
    * Chai
* Useful links
* Contributing

## What is it?

Amaze TDD (test-driven development) is a library that makes it easy to write functional front-end tests that rely on a server or specific URL. It includes Selenium, a tool for automating web browser interactions, the Mocha test runner, and the Chai assertion library. It has been built to accommodate Chrome and PhantomJS (a headless browser).

The main difference between using this package and, say, Mocha by itself, is that Mocha would only run locally and requires HTML fixture files and separate scripts. This package is ideal for testing on remote servers where databases or dynamic content comes in to play.

## Requirements

This package should work on any system that can run Node. That being said, you will need Node and NPM installed. Because of recent updates to Node and the way dependencies are handled, we recommend Node version 4.2 to 4.6 for the time being. Check out [NVM](https://github.com/creationix/nvm) if you need various versions of Node installed.

You will also need Chrome installed.

Lastly, if you wish to use PhantomJS as your `phantomjs` testing environment, you'll need that installed globally. You can do that with:

`npm install -g phantomjs`

### Mac

Lucky for you, things just work! Carry on.

### Windows

Ensure that Chrome and the included `chromedriver` is in your `PATH`. We recommend copying (not moving) the included drivers to a permanent location on your computer and using that location as your `PATH` reference. This will ensure that no matter where your test and project folder are, it will work.

### Linux

Lucky for you, things just work! Carry on.

## Browsers

Amaze TDD is setup to run tests with Chrome and PhantomJS, the former being more geared towards development, and the latter being more geared towards production (as it's a bit faster since no browser windows need to spawn).

There are future plans to include Firefox and IE/Edge as well, but for now, those aren't available.

## The `Amaze` Object

We've made it extremely easy to write tests, but if you're curious and want to write advanced tests, the `Amaze` object consists of the following properties:

* `Webdriver` Selenium root package
* `Client` the client object for the browser
* `By` used to located elements on a page (if you're not fond of jQuery)
* `Until` may be used to set waits and webdriver expectations for async tests  
* `Chrome` the chromedriver root package
* `Mocha` our pre-configured Mocha instance (see config in `amaze.js`)
* `Expect` Chai expectation methods (most commonly used)
* `Assert` Chai assertion methods
* `$` a limited jQuery library useful for easier HTML object/element locating

Within your tests, you can access these properties via `Amaze.[property]`.

## `.amazerc` configuration options

You can optionally include an `.amazerc` file in the root directory of your project that exports a dictionary of settings to make test writing easier. This file may contain URLs, usernames, passwords, or any other piece of data that is used repeatedly. This helps reduce errors and typos and provides a single point of configuration for your suite.

We've included a sample file for you along with example use in the demo test, but for brevity, you would access your variables as such:

`Amaze.Configs.url` where `url` would be the configuration value in the `.amazerc` file. You may use things like passwords, usernames, or any other piece of data that is used repeatedly so you only have to maintain a single instance.

## Writing a test

The idea behind test-driven development is to write a test that has expectations, and then write your overlays. Mocha is, by default, asynchronous and promise-based, so write your tests accordingly. We've included a demo test in this package, but the main setup is as follows:

Include the global Amaze object:
```
var Amaze = require('amaze-tdd');
```

Instantiate the browser object for use with testing. The below will set configurations for Chrome. If you use PhantomJS, irrelevant configs are ignored.
```
var Browser;
...
Browser = new Amaze.Client.Build();
```

Begin your TDD suite:
```
suite('name of your suite', function() {

});
```

You may use vanilla JavaScript or our limited [selection of jQuery methods](https://npmjs.com/package/selenium-query) in your tests. You will be working with WebObjects, jQuery objects, and promises.

### TDD interface

The TDD interface consists of the following:

* `suite()`
* `suiteSetup()`
* `test()`
* `suiteTeardown()`
* `setup()`
* `teardown()`

#### Suite

Suite defines the test as a whole and acts as a group for the individual tests. They may be nested for better organization.

```
suite('section', function() {
    suite('subsection', function() {
        // tests
    });
});
```

#### suiteSetup

Use this within a given Suite to set configurations or variables available to that entire Suite.

#### test

The actual, individual test cases within each Suite. They generally return a condition or promise.

#### suiteTeardown

Much like suiteSetup, this unloads or unsets anything configured for a given Suite.

#### setup

Similar to suiteSetup, but generally used within sub-suites.

#### teardown

Much like setup, this unloads or unsets anything configured in setup.

### Using jQuery

The `Amaze` object contains a [limited package of jQuery](https://npmjs.com/package/selenium-query) for use within tests. It is accessible via `Amaze.$` and can then be used similarly to how you would normally use jQuery. For example:

```
Amaze.$(Browser).find('#el') ...
```

### Chai

The object also contains the entire Chai library separated into two main sub-libraries.

#### expect

The most commonly used assertion. Methods are chainable.

```
Amaze.Expect(this).to.equal(that)
```

#### assert

Another assertion pattern. Methods are **not** chainable.

```
Amaze.Assert('foo' !== 'bar')
```

## Useful links

* [Chai expect](http://chaijs.com/api/bdd/) and [Chai assert](http://chaijs.com/api/assert/)
* [Mocha](http://mochajs.org/)
* Searchable docs for various [Selenium Webdriver methods](http://seleniumhq.github.io/selenium/docs/api/javascript/module/selenium-webdriver/)
* Information on the [jQuery plugin](https://www.npmjs.com/package/selenium-query) used in this suite
* [Documentation](docs/index.md) on writing good tests
* [Wiki](https://bitbucket.org/dmusser/amaze-tdd/wiki/Home) with even more information on this suite and TDD

## Contributing

Easy. Just make a feature branch and open a pull request. Please note that we follow proper [Gitflow](https://www.atlassian.com/git/tutorials/comparing-workflows/feature-branch-workflow) practices:

* **master** is locked and always synced with the latest release
* **develop** is where the action is at and is likely ahead of master; you start here
* **feature** branches contain individual improvements, or features; branched from develop
* **hotfix** is branched from master to get quick fixes directly back to master; release branches are made when necessary
* **release** contains features between the latest release and HEAD

